# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['__init__']
setup_kwargs = {
    'name': 'pyhtnorm',
    'version': '0.1.0',
    'description': 'Fast Simulation of Hyperplane-Truncated Multivatiate Normal Distributions',
    'long_description': '# htnorm\n\nThis repo provides a C implementation of a fast and exact sampler from a \nmultivariate normal distribution (MVN) truncated on a hyperplane as described [here][1]\n\nthis repo implements the following from the paper:\n\n- efficient Sampling from a MVN truncated on a hyperplane: \n\n    ![hptrunc](https://latex.codecogs.com/svg.latex?%5Cmathbf%7Bx%7D%20%5Csim%20%5Cmathcal%7BN%7D_%7B%5Cmathcal%7BS%7D%7D%28%5Cmathbf%7B%5Cmu%7D%2C%20%5Cmathbf%7B%5CSigma%7D%29%3B%20%5Chspace%7B2mm%7D%20%5Cmathcal%7BS%7D%20%3D%20%5C%7B%5Cmathbf%7Bx%7D%20%3A%20%5Cmathbf%7BG%7D%5Cmathbf%7Bx%7D%20%3D%20%5Cmathbf%7Br%7D%5C%7D%2C%20%5Cmathbf%7BG%7D%20%5Cin%20%5Cmathcal%7BR%7D%5E%7Bk_2%20%5Ctimes%20k%7D%2C%20rank%28%5Cmathbf%7BG%7D%29%20%3D%20k_2%20%3C%20k)\n\n- efficient sampling from a MVN with a stuctured precision matrix: \n\n    ![struc](https://latex.codecogs.com/svg.latex?%5Cmathbf%7Bx%7D%20%5Csim%20%5Cmathcal%7BN%7D%5C%5B%5Cmathbf%7B%5Cmu%7D%2C%20%28%5Cmathbf%7BA%7D%20&plus;%20%5Cmathbf%7B%5CPhi%7D%5ET%5Cmathbf%7B%5COmega%7D%5Cmathbf%7B%5CPhi%7D%29%5E%7B-1%7D%5C%5D%3B%20%5Chspace%7B2mm%7D%20%5Cmathbf%7B%5CPhi%7D%20%5Cin%20%5Cmathcal%7BR%7D%5E%7Bn%20%5Ctimes%20p%7D%2C%20%5Cmathbf%7B%5COmega%7D%20%5Cin%20%5Cmathcal%7BR%7D%5E%7Bn%20%5Ctimes%20n%7D%2C%20%5Cmathbf%7BA%7D%20%5Cin%20%5Cmathcal%7BR%7D%5E%7Bp%20%5Ctimes%20p%7D)\n\n- efficent sampling frfom a MVN with a structured precision and mean:\n\n    ![strucmean](https://latex.codecogs.com/svg.latex?%5Cmathbf%7Bx%7D%20%5Csim%20%5Cmathcal%7BN%7D%5CBig%5C%5B%28%5Cmathbf%7BA%7D%20&plus;%20%5Cmathbf%7B%5CPhi%7D%5ET%5Cmathbf%7B%5COmega%7D%5Cmathbf%7B%5CPhi%7D%29%5E%7B-1%7D%5Cmathbf%7B%5CPhi%7D%5ET%5Cmathbf%7B%5COmega%7D%5Cmathbf%7Bt%7D%2C%20%28%5Cmathbf%7BA%7D%20&plus;%20%5Cmathbf%7B%5CPhi%7D%5ET%5Cmathbf%7B%5COmega%7D%5Cmathbf%7B%5CPhi%7D%29%5E%7B-1%7D%5CBig%5C%5D%3B%20%5Chspace%7B2mm%7D%20%5Cmathbf%7B%5COmega%7D%20%5Cin%20%5Cmathcal%7BR%7D%5E%7Bn%20%5Ctimes%20n%7D%2C%20%5Cmathbf%7BA%7D%20%5Cin%20%5Cmathcal%7BR%7D%5E%7Bp%20%5Ctimes%20p%7D)\n\nThe algorithms implemented have the following practical applications:\n- Topic models when unknown parameters can be interpreted as fractions.\n- Admixture models\n- discrete graphical models\n- Sampling from posterior distribution of an Intrinsic Conditional Autoregressive prior [icar][8]\n- Sampling from posterior conditional distributions of various bayesian regression problems.\n\n\n## Dependencies\n\n- a C compiler that supports the C99 standard or later\n- an installation of BLAS and LAPACK that exposes its C interface via the headers `<cblas.h>` and `<lapacke.h>`\n(e.g openBLAS).\n\n\n## Usage\n\nBuilding a shared library of `htnorm` can be done with the following:\n```bash\n$ cd src/\n# optionally set path to CBLAS and LAPACKE headers using INCLUDE_DIRS environmental variable\n$ export INCLUDE_DIRS="some/path/to/headers" \n# optionally set path to BLAS installation shared library\n$ export LIBS_DIR="some/path/to/library/"\n# optionally set the linker flag for your BLAS installation (e.g -lopenblas)\n$ export LIBS=<flag here>\n$ make lib\n```\nAfterwards the shared library will be found in a `lib/` directory of the project root,\nand the library can be linked dynamically via `-lhtnorm`.\n\nThe puplic API exposes the samplers through the function declarations\n- `int htn_hyperplane_truncated_mvn(rng_t* rng, const ht_config_t* conf, double* out)`\n- `int htn_structured_precision_mvn(rng_t* rng, const sp_config_t* conf, double* out)`\n\nThe details of the parameters are documented in ther header files ["htnorm.h"][4].\n\nRandom number generation is done using [PCG64][2] or [Xoroshiro128plus][3] bitgenerators. \nThe API allows using a custom generator, and the details are documented in the header file \n["rng.h"][5].\n\n## Examples\n```C\n#include "htnorm.h"\n\nint main ()\n{\n    ...\n\n    // instantiate a random number generator\n    rng_t* rng = rng_new_pcg64();\n    ht_config_t config;\n    config.g = ...; // G matrix\n    config.gnrow = ...; // number of rows of G\n    config.gncol = ...; // number of columns of G\n    cofig.r = ...; // r array\n    config.mean = ...; // mean array\n    config.cov = ...; // the convariance matrix\n    confi.diag = ...; // whether covariance is diagonal\n\n    double* samples = ...; // array to store the samples\n    // now call the sampler\n    int res_info = htn_hyperplane_truncated_mvn(rng, &config, samples);\n\n    // res_info contains a number that indicates whether sampling failed or not.\n\n    ...\n\n    // finally free the RNG pointer at some point\n    rng_free(rng);\n\n    ...\n    return 0;\n}\n```\n\n## Python API\n\nA high level python interface to the library is also provided. Linux users can \ninstall it using wheels via pip (thus not needing to worry about availability of C libraries),\n```bash\npip install pyhtnorm\n```\nWheels are not provided for MacOS. To install via pip, one can run the following commands:\n```bash\n#set the path to BLAS installation headers\nexport INCLUDE_DIR=<path/to/headers>\n#set the path to BLAS shared library\nexport LIBS_DIR=<some directory>\n#set the name of the BLAS shared library (e.g. "openblas")\nexport LIBS=<lib name>\n# finally install via pip so the compilation and linking can be done correctly\npip install pyhtnorm\n```\nAlternatively, one can install it from source. This requires an installation of [poetry][7] and the following shell commands:\n\n```bash\n$ git clone https://github.com/zoj613/htnorm.git\n$ cd htnorm/\n$ poetry install\n# add htnorm to python\'s path\n$ export PYTHONPATH=$PWD:$PYTHONPATH\n```\n\nBelow is an example of how to use htnorm in python to sample from a multivariate\ngaussian truncated on the hyperplane ![sumzero](https://latex.codecogs.com/svg.latex?%5Cmathbf%7B1%7D%5ET%5Cmathbf%7Bx%7D%20%3D%200) (i.e. making sure the sampled values sum to zero)\n\n```python\nfrom pyhtnorm import HTNGenerator\nimport numpy as np\n\nrng = HTNGenerator()\n\n# generate example input\nk1 = 1000\nk2 = 1\nnpy_rng = np.random.default_rng()\ntemp = npy_rng.random((k1, k1))\ncov = temp @ temp.T + np.diag(npy_rng.random(k1))\nG = np.ones((k2, k1))\nr = np.zeros(k2)\nmean = npy_rng.random(k1)\n\nsamples = rng.hyperplane_truncated_mvnorm(mean, cov, G, r)\n# verify if sampled values sum to zero\nprint(sum(samples))\n\n# alternatively one can pass an array to store the results in\nout = np.empty(k1)\nrng.hyperplane_truncated_mvnorm(mean, cov, G, r, out=out)\n# verify\nprint(out.sum())\n```\n\nFor more details about the parameters of the `HTNGenerator` and its methods,\nsee the docstrings via python\'s `help` function.\n\nThe python API also exposes the `HTNGenerator` class as a Cython extension type\nthat can be "cimported" in a cython script.\n\n\n## TODO\n\n- Add an `R` interface to the library.\n\n\n## Licensing\n\n`htnorm` is free software made available under the BSD-3 License. For details\nsee the [LICENSE][6] file.\n\n\n## References\n- Cong, Yulai; Chen, Bo; Zhou, Mingyuan. Fast Simulation of Hyperplane-Truncated \n   Multivariate Normal Distributions. Bayesian Anal. 12 (2017), no. 4, 1017--1037. \n   doi:10.1214/17-BA1052.\n- Bhattacharya, A., Chakraborty, A., and Mallick, B. K. (2016). \n  “Fast sampling with Gaussian scale mixture priors in high-dimensional regression.” \n  Biometrika, 103(4):985. \n\n\n[1]: https://projecteuclid.org/euclid.ba/1488337478\n[2]: https://www.pcg-random.org/\n[3]: https://en.wikipedia.org/wiki/Xoroshiro128%2B\n[4]: https://github.com/zoj613/htnorm/blob/main/include/htnorm.h \n[5]: https://github.com/zoj613/htnorm/blob/main/include/rng.h\n[6]: https://github.com/zoj613/htnorm/blob/main/LICENSE\n[7]: https://python-poetry.org/docs/pyproject/\n[8]: https://www.sciencedirect.com/science/article/abs/pii/S1877584517301600\n',
    'author': 'Zolisa Bleki',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/zoj613/htnorm/',
    'py_modules': modules,
    'python_requires': '>=3.6.1,<4.0.0',
}
from build import *
build(setup_kwargs)

setup(**setup_kwargs)
